package http

import (
	"errors"
	"framework_v2/internal/app/auth"
	"framework_v2/internal/app/config"
	"framework_v2/internal/app/facade"
	"framework_v2/internal/app/helper"
	"framework_v2/internal/app/jwks"
	"github.com/gin-gonic/gin"
	"github.com/mitchellh/mapstructure"
	"net/http"
	"strings"
)

const AnonymousUser = "anonymous"

var (
	ErrNotValidToken  = errors.New("无效的 JWT 令牌。")
	ErrJWTFormatError = errors.New("JWT 格式错误。")
	ErrNotBearerType  = errors.New("不是 Bearer 类型。")
	ErrEmptyResponse  = errors.New("我们的服务器返回了空请求，可能某些环节出了问题。")
)

// DIJWTAuth 用于注入到方法签名中。我觉得下面的代码以后可以优化。
func DIJWTAuth(c *gin.Context) *auth.User {
	var sub = AnonymousUser
	var jwtIdToken = &auth.User{}

	if config.Config.DebugMode.Enable {
		jwtIdToken.Token.Sub = sub
	} else {
		// get authorization header
		authorization := c.Request.Header.Get("Authorization")

		if authorization == "" {
			helper.ResponseError(c, http.StatusUnauthorized, ErrJWTFormatError)
			return nil
		}

		authSplit := strings.Split(authorization, " ")
		if len(authSplit) != 2 {
			helper.ResponseError(c, http.StatusUnauthorized, ErrJWTFormatError)
			return nil
		}

		if authSplit[0] != "Bearer" {
			helper.ResponseError(c, http.StatusUnauthorized, ErrNotBearerType)
			return nil
		}

		token, err := jwks.ParseJWT(authSplit[1])
		if err != nil {
			helper.ResponseError(c, http.StatusUnauthorized, ErrJWTFormatError)
			return nil
		}
		sub, err = token.Claims.GetSubject()
		if err != nil {
			helper.ResponseError(c, http.StatusUnauthorized, ErrNotValidToken)
			return nil
		}

		err = mapstructure.Decode(token.Claims, &jwtIdToken.Token)
		if err != nil {
			facade.Logger.Error("Failed to map token claims to JwtIDToken struct.\nError: " + err.Error())
			helper.ResponseError(c, http.StatusUnauthorized, ErrNotValidToken)
			return nil
		}
	}

	return jwtIdToken
}
